/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#include "../lib/piconf/piconfm/tag/pi_tag.hpp"
#include "../lib/interim/interim.hpp"
#include "../lib/ape/ape.hpp"

#include <iostream>
#include <map>
#include <deque>
#include <vector>
#include <string>
#include <string.h>

//#include "../lib/pibox/mixed.hpp"
int main(int argc, char** argv) {/*
	try {
		typedef pibox::mixed box_t;
		box_t box(5, 10);
		box.push_back("1", 2);
		box.push_back("2", 2);
		box.push_back("3", 2);
		box.push_back("4", 2);
		box.push_back("5", 2);
		box.pop_front();
		box.push_back("6", 2);
		size_t front, back;
		box.data_left(front, back);
		std::cout << "front_left: " << front << "; back_left: " << back << std::endl;
		//for (size_t i = 0; i < box.item_used(); ++i)
		//	std::cout << box[-(1+i)].data << std::endl;
		//for (box_t::reverse_iterator it = box.rbegin(); it != box.rend(); it+=1)
		//	std::cout << it->data << std::endl;
		box_t::iterator it = box.begin()+box.item_used();
		if (it > box.begin() and &*it < &*box.begin() and box.continious_front(3))
			std::cout << box.difference(&*it, &*box.begin()) << std::endl;
	}
	catch (std::string& error) {
		std::cerr << error << std::endl;
	}
	return 0;*/
	interim terminal;
	try {
		std::string file = piconf::parse_args(piconf::name, argc, argv);
		timespec span = {0, 10000000};
		VIO::TCP timer;
		VIO::IO::timer = &timer;
		VIO::TCP::forkTimer(&timer, span);
		VIO::IO::IOS mask;
		mask.insert(VIO::IO::timer);
		mask.insert(&terminal.in);
		ape::tree tree;
		pugi::xml_node root = tree.parse(file.c_str(), PICONF_ROOT);
		ape::resolve_fpath(root, PICONF_ROOT);
		ape::resolve_xpath(root);
		std::deque<piconf::pi_tag> pi_box;
		for (pugi::xml_node_iterator r = root.begin(); r != root.end() and r->type() == 2; r = ++r) {
			if (!strcmp(r->name(), "pi")) {
				pi_box.push_back(piconf::pi_tag(*r));
				pi_box.back().setup();
			}
		}
		if (root.attribute("dump").as_bool()) {
			tree.doc.save(std::cout);
			return piconf::error.get();
		}
		bool silent = root.attribute("silent").as_bool();
		if (!silent) {
			terminal.topstart(root.attribute("topstart").as_bool(true));
			terminal.automove(root.attribute("automove").as_bool(true));
			terminal.open();
		}
		piconf::lines_t lines;
		for (size_t i = 0; i < pi_box.size(); ++i) {
			bool quit = false;
			bool halt = false;
			lines.push_back("Processing instruction ("+piconf::to_string(i+1)+"/"+piconf::to_string(pi_box.size())+")...");
			if (piconf::error.get()) {
				lines.push_back(" Aborted due to previous errors.");
				terminal.lines.clear();
				terminal.lines.insert(terminal.lines.begin(), lines.begin(), lines.end());
				if (!silent) {
					terminal.update();
				}
				break;
			}
			for (bool undone = true; undone;) {
				interim::nkeys_t nkeys = terminal.parseKeys();
				for (interim::nkeys_t::iterator nkey = nkeys.begin(); nkey != nkeys.end(); ++nkey) {
					if (*nkey == 'p') {
						halt = !halt;
						if (halt) {
							VIO::IO::mask(&mask);
							terminal.lines.push_back("Paused by user.");
							terminal.update();
						}
						else
							VIO::IO::mask();
					}
					if (*nkey == 'q') {
						quit = true;
					}
				}
				if (quit) {
					if (!silent) {
						terminal.lines.push_back("Interrupted by user.");
						terminal.update();
					}
					break;
				}
				if (!halt) {
					if (!silent) {
						terminal.lines.clear();
						terminal.lines.insert(terminal.lines.begin(), lines.begin(), lines.end());
						pi_box[i].status(terminal.lines);
						terminal.update();
					}
					if (timer.schedule()) {
						timespec time = timer.getLastTime();
						pi_box[i].schedule(&span, &time);
					}
					undone = pi_box[i].process();
					if (!undone)
						if (!silent) {
							terminal.lines.clear();
							terminal.lines.insert(terminal.lines.begin(), lines.begin(), lines.end());
							pi_box[i].status(terminal.lines);
							terminal.lines.push_back((std::string)(piconf::error.get() ? " Failed" : " Done")+".");
							terminal.update();
						}
				}
				else {
					terminal.update();
					VIO::IO::process();
				}
			}
			if (quit)
				break;
			lines = terminal.lines;
		}
		VIO::IO::mask(&mask);
		if (!silent)
			for (bool undone = true; undone; undone = VIO::IO::process()) {
				interim::nkeys_t nkeys = terminal.parseKeys();
				for (interim::nkeys_t::iterator nkey = nkeys.begin(); nkey != nkeys.end(); ++nkey) {
					if (*nkey == KEY_ESC)
						return piconf::error.get();
					terminal.update();
				}
			}
	}
	catch (VIO::IO::Error& error) {
		/* DEBUG:
		 * 
		 * When resizing terminal, piconfm exits here with 128.
		 * 
		 * GDB says there is no signal thrown (like SIGWINCH).
		 * 
		 * error.what() = "Timer disconnected"
		 * ... very strange, so we loose our socketpair as soon as the
		 * terminal is resized?!
		 *
		 * But a resize was possible! Will test old commits and maybe
		 * old ncurses versions ...
		 */
		piconf::error.set(piconf::error_t::SYSCALL_ERROR);
		terminal.close();
		std::cerr << error.what() << std::endl;
	}
	catch (std::bad_alloc& error) {
		piconf::error.set(piconf::error_t::SYSCALL_ERROR);
		terminal.close();
		std::cerr << "No RAM left" << std::endl;
	}
	catch (std::string& error) {
		if (!piconf::error.get())
			piconf::error.set(piconf::error_t::PARSING_ERROR | piconf::error_t::PICONFM_ERROR);
		terminal.close();
		std::cerr << error << std::endl;
	}
	return piconf::error.get();
}
