/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _APE_H
#define _APE_H

#include "../pugixml/pugixml.hpp"

#include <vector>
#include <string>
#include <wordexp.h>
#include <stdlib.h>

namespace ape {
	std::vector<std::string> explore(pugi::xml_node node);
	std::string implode(std::vector<std::string> subject);
	ssize_t level(const pugi::xml_node* subject, const pugi::xml_node* reference = 0, bool* related = 0);
	bool has_ancestor(pugi::xml_node subject, pugi::xml_node reference, ssize_t offset = 0);
	std::vector<std::string> perform_word_expansion(std::string word, int flags = WRDE_NOCMD | WRDE_UNDEF);
	void resolve_fpath(pugi::xml_node node, const char* root = 0, const char* tag = "fpath");
	void resolve_xpath(pugi::xml_node node, const char* tag = "xpath");
	void merge_twins(pugi::xml_node node);
	void inherit_attributes(pugi::xml_node node);
	class tree;
}

class ape::tree {
	std::string xml;
	pugi::xml_parse_result result;
public:
	static std::string get_file_contents(std::string file, bool cin = true);
	pugi::xml_document doc;
	tree();
	tree(const char* source, const char* root = 0, bool file = true, bool cin = true);
	pugi::xml_node parse(const char* source, const char* root = 0, bool file = true, bool cin = true);
};

#endif
