/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_DAQ_TAG_H
#define _PICONF_DAQ_TAG_H

#include "data_tag.hpp"
#include "../../../vio/File.hpp"
#include "../../../pibox/mixed.hpp"
#include <map>
#include <vector>
#include <deque>
#include <string>
#include <utility>
#include <inttypes.h>

namespace piconf {
	class daq_tag;
}

class piconf::daq_tag : public data_tag {
	static const size_t MAX_DATA_SIZE = 1024*1024*1024;
	static const size_t MAX_ITEM_SIZE = MAX_DATA_SIZE/100;
	static const size_t DEF_DATA_SIZE = 1024*1024*10;
	static const size_t DEF_ITEM_SIZE = DEF_DATA_SIZE/100;
	size_t item_size, item_done;
	size_t data_size, data_done;
	typedef uint8_t daq_state_t;
	daq_state_t daq_state;
	static const daq_state_t
	READBACK = 0x01,
	FILE_RB  = 0x02,
	DAQ      = 0x04,
	IDLE     = 0x08,
	FAIL     = 0x10,
	DONE     = 0x20;
	class rb_t {
	public:
		std::string
		path,
		name,
		node,
		data;
	};
	rb_t rb;
	class file_t {
	public:
		VIO::File file;
		typedef uint8_t state_t;
		static const state_t
		WRITING = 0x01,
		DONE    = 0x02;
		state_t state;
		std::string* data;
		file_t();
	};
	typedef std::map<io_tag*, file_t> file_map_t;
	file_map_t file_map;
	typedef std::deque<file_t> file_box_t;
	file_box_t file_box;
	class ring_t {
	public:
		uint32_t head[4];
		pibox::mixed* ring;
		typedef uint32_t daq_state_t;
		daq_state_t daq_state;
		static const daq_state_t
		INVALID = 0x0001,
		ACQUIRE = 0x0002,
		READOUT = 0x0004,
		DISCARD = 0x0008,
		EMPTIED = 0x0010,
		OVERRUN = 0x0020,
		HOOK_UP = 0x0040,
		READING = 0x0080,
		WRITING = 0x0100,
		SYNCING = 0x0200,
		STALLED = 0x0400,
		UPGRADE = 0x0800,
		CEASING = 0x1000;
		ring_t();
		pibox::mixed::reverse_iterator i_item, o_item;//push_front... and then shrink front to remove size and item_done and data_done. hm, that what cause buffer to be bigger that neccessary, good? bad? GOOD IN THE END, less total memory usage, ring should just be big enough for such io stuff!!!!!!! do it the same for axi if streaming and maybe also for axi gpio, so reverse ring on slave...
		struct {
			size_t item_used;
			size_t item_size;
			size_t data_used;
			size_t data_size;
		} m, s;
		size_t item_done, data_done;
	};
	typedef std::map<io_tag*, ring_t> ring_map_t;
	ring_map_t ring_map;
	inline static std::string file(std::string id, pugi::xml_node node) {
		pugi::xml_node root = node.parent();
		return (std::string)node.text().get()+"/"+root.attribute("prefix").value()+id+root.attribute("suffix").value();
	}
public:
	daq_tag(pugi::xml_node node);
	void setup();
	bool instruct(io_tag* io, bool fail = false, bool exit = false);
	void parse_response(io_tag* io, pugi::xml_node node);
	bool process();
	void status(lines_t& lines);
	~daq_tag();
};

#endif
