/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_DATA_TAG_H
#define _PICONF_DATA_TAG_H

#include "../piconfm.hpp"
#include "../../../pugixml/pugixml.hpp"
#include <inttypes.h>
#include <deque>
#include <string>
#include <utility>
#include <map>
#include <vector>

class piconf::data_tag : public tag {
protected:
	typedef std::pair<char*, size_t> data_t;
	typedef std::deque<data_t> data_box_t;
	typedef uint16_t io_state_t;
	struct io_t {
		lines_t result;
		io_state_t state;
		static const io_state_t
		INSTRUCTED_RESPONSE_SIZE = 0x01,
		INSTRUCTED_RESPONSE_XML  = 0x02,
		RECEIVED_RESPONSE        = 0x04,
		PARSED_RESPONSE          = 0x08,
		VALID_RESPONSE           = 0x10,
		EXIT                     = 0x20;
		uint32_t size;
		data_box_t data_box;
		io_t();
	};
	typedef std::map<io_tag*, io_t > io_box_t;
	io_box_t io_box;
	data_tag(pugi::xml_node node);
public:
	void subscribe(std::deque<io_tag*>& io_box, bool inherit);
	virtual bool instruct(io_tag* io, bool fail = false, bool exit = false);
	virtual void parse_response(io_tag* io, pugi::xml_node node);
	virtual void result(io_tag* io, lines_t& lines);
	virtual bool complete();
	bool process();
	void setup();
	std::string root();
	virtual ~data_tag();
};

#endif
