/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_TCP_TAG_H
#define _PICONF_TCP_TAG_H

#include "io_tag.hpp"
#include "../../../vio/TCP.hpp"
#include <string>
#include <vector>
#include <time.h>

namespace piconf {
	class tcp_tag;
}

class piconf::tcp_tag : public io_tag {
	static const uint8_t
	CONNECTING = 0x01,
	INSTRUCT   = 0x02,
	DONE       = 0x04,
	SLEEP      = 0x08,
	CONNECTED  = 0x10,
	FAIL       = 0x20;
	VIO::TCP tcp;
	uint8_t tcp_state;
	int code;
	std::string host, service;
	static const timespec delay;
	timespec timeout, reconnect_delay, reconnect_delay_begin;
	size_t reconnect_tries;
	bool reconnect_limit;
	size_t reconnect_count;
public:
	bool awake();
	tcp_tag(pugi::xml_node node);
	void setup();
	void status(lines_t& lines);
	void handle(int code, std::string what);
	virtual void schedule(const timespec* interval = 0, const timespec* realtime = 0, const timespec* accuracy = 0);
	virtual bool process();
	std::string id();
	~tcp_tag();
};

#endif
