/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_S_H
#define _PICONF_S_H

#include "../piconf/piconf.hpp"
#include "../../vio/AXI.hpp"
#include "../../ape/ape.hpp"
#include "../../pugixml/pugixml.hpp"

#include <string>
#include <map>

namespace piconf {
	extern name_t name;
	const size_t MAX_AXI_GPIO_SIZE = 2048;
	template <typename number_t>
	std::string in_range(number_t what, number_t min, number_t max);
	template <typename number_t>
	std::string get_uint(const void* data, number_t& what, size_t offset = 0, size_t bits = 1, size_t min = 0, size_t max = 0);
	template <typename number_t>
	std::string set_uint(void* data, number_t what, size_t offset = 0, size_t bits = 1, size_t min = 0, size_t max = 0);
	int shell(line_t& output, line_t input, chunk_t size = 512);
	typedef std::deque<VIO::AXI> axi_box_t;
	typedef std::deque<VIO::AXI*> axi_ptr_box_t;
	typedef std::map<std::string, VIO::AXI*> axi_map_box_t;
	typedef std::deque<axi_ptr_box_t> axi_cat_box_t;
	typedef std::map<std::string, size_t> axi_buf_box_t;
	void axi_gpio(pugi::xml_node node, axi_map_box_t& zdx_box, axi_buf_box_t& buf_box, std::ofstream& log);
}

template <typename number_t>
static std::string piconf::in_range(number_t what, number_t min, number_t max) {
	if (min > what or (max and what > max))
		return "Value of "+to_string(what)+" is not in range from "+to_string(min)+" to "+to_string(max);
	return "";
}

template <typename number_t>
static std::string piconf::get_uint(const void* data, number_t& what, size_t offset = 0, size_t bits = 1, size_t min = 0, size_t max = 0) {
	what = 0;
	for (number_t i = 0; i < bits; ++i) {
		size_t e = i+offset;
		if (*((const char*)data+e/8) & (1 << (e%8)))
			what |= 1 << i;
		else
			what &= ~(1 << i);
	}
	return in_range<number_t>(what, min, max);
}

template <typename number_t>
static std::string piconf::set_uint(void* data, number_t what, size_t offset = 0, size_t bits = 1, size_t min = 0, size_t max = 0) {
	for (number_t i = 0; i < bits; ++i) {
		size_t e = i+offset;
		if (what & (1 << i))
			*((char*)data+e/8) |= 1 << (e%8);
		else
			*((char*)data+e/8) &= ~(1 << (e%8));
	}
	return in_range<number_t>(what, min, max);
}

#include "tag/tcp_tag.hpp"

#endif
