/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_DAQ_TAG_H
#define _PICONF_DAQ_TAG_H

#include "../piconfs.hpp"
#include "../../../pibox/mixed.hpp"
#include "data_tag.hpp"

namespace piconf {
	class daq_tag;
}

class piconf::daq_tag : public data_tag {
	static const size_t MAX_RING_SIZE = 256*1024*1024;
	static const size_t MAX_ITEM_SIZE = MAX_RING_SIZE/100;
	static pibox::mixed ring;
	static size_t item_sync, item_size, ring_size;
	static size_t r_item_done, i_item_done, o_item_done;
	static pugi::xml_node tpl;
	static char* data;
	static axi_map_box_t zdx_box;
	static size_t data_size;
	typedef uint16_t daq_state_t;
	static daq_state_t daq_state;
	static const daq_state_t
	INVALID = 0x0001,
	ACQUIRE = 0x0002,
	READOUT = 0x0004,//if not, send item_size and data_size and !what! though to update master screen
	DISCARD = 0x0008,//query: discard = 1; local: discard -> 0; remote: only applied if write done.
	EMPTIED = 0x0010,
	OVERRUN = 0x0020,
	HOOK_UP = 0x0040,
	READING = 0x0080,
	WRITING = 0x0100,
	SYNCING = 0x0200,
	STALLED = 0x0400,
	UPGRADE = 0x0800,
	CEASING = 0x1000;//query: acquire = 0, ceasing = 1; value: ceasing = 0 if buffer empty
	static void hook();
	static hooked_t hooked;
	static timespec activate;
public:
	static void construct(std::ofstream& log, pugi::xml_node tpl, hooked_t*& hooked, timespec activate);
	daq_tag(std::ofstream& log, pugi::xml_node tpl, pugi::xml_node tag, VIO::IO* io);
	bool hook_me_up();
	bool process();
	static void destruct();
};

#endif
