/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_DATA_TAG_
#define _PICONF_DATA_TAG_

#include "data_tag.hpp"
#include "../piconfs.hpp"
#include <sstream>
#include <arpa/inet.h>
#include <string.h>

piconf::data_tag::hooked_t::ios_t piconf::data_tag::queued;

piconf::data_tag::data_tag(std::ofstream& log, pugi::xml_node tag, VIO::IO* io)
: tag() {
	this->log = &log;
	this->tag = tag;
	this->io = io;
	state = 0;
	value = 0;
}

void piconf::data_tag::respond() {
	std::ostringstream os;
	tag.print(os);
	xml = os.str();
	size = htonl(xml.size()+1);
	delete[] value;
	value = new char[(sizeof size)+xml.size()+1];
	*((uint32_t*)value) = size;
	memcpy(value+sizeof size, xml.c_str(), xml.size()+1);
	state |= RESPOND;
}

bool piconf::data_tag::hook_me_up() {
	return false;
}

piconf::data_tag::hooked_t::ios_t& piconf::data_tag::hook_me_down() {
	return queued;
}

bool piconf::data_tag::process() {
	if (state & RESPOND) {
		state &= ~RESPOND;
		state |= RESPONDING;
		//io->push(&size, sizeof size);
		//io->push(xml.c_str(), xml.size()+1);
		io->push(value, (sizeof size)+xml.size()+1);
		return true;
	}
	else
	if (state & RESPONDING) {
		if (!io->hasToWrite()) {
			state &= ~RESPONDING;
			*log << time() << "Response sent" << std::endl;
		}
		else
			return true;
	}
	return false;
}

piconf::data_tag::~data_tag() {
	delete[] value;
}

#endif
