/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _PICONF_SHELL_TAG_
#define _PICONF_SHELL_TAG_

#include "shell_tag.hpp"
#include "../piconfs.hpp"
#include <string.h>
#include <stdlib.h>

piconf::shell_tag::shell_tag(std::ofstream& log, pugi::xml_node tpl, pugi::xml_node tag, VIO::IO* io)
: data_tag(log, tag, io) {
	this->tpl = tpl;
	shell_state = 0;
	int code = 0;
	pugi::xml_attribute what = tag.attribute("what");
	input = tpl.child(what.value());
	if (!input) {
		tag.text().set(((std::string)"Shell input '"+what.value()+"' unknown").c_str());
		code |= error_t::XML_TAG_ERROR | error_t::PICONFM_ERROR;
		shell_state |= INVALID;
	}
	if (!(shell_state & INVALID)) {
		if (input.attribute("no_return").as_bool())
			shell_state |= NO_RETURN;
		if (shell_state & NO_RETURN)
			tag.text().set("Shell is processing");
		else {
			std::string output;
			int result = shell(output, input.text().get());
			log << time() << "Shell result: " << result << std::endl;
			if (result)
				code |= error_t::SYSCALL_ERROR | error_t::PICONFS_ERROR;
			tag.text().set(output.c_str());
			if (!tag.attribute("result"))
				tag.append_attribute("result");
			tag.attribute("result").set_value(to_string(result).c_str());
		}
	}
	if (code) {
		if (!tag.attribute("error"))
			tag.append_attribute("error");
		tag.attribute("error").set_value(to_string(code).c_str());
	}
	respond();
	log << time() << "Parsed shell_tag" << std::endl;
}

bool piconf::shell_tag::process() {
	data_tag::process();
	if (!(state & RESPONDING)) {
		if (shell_state & INVALID) {
			*log << time() << "Shell invalid" << std::endl;
			return false;
		}
		if (shell_state & NO_RETURN) {
			std::string output;
			int result = shell(output, input.text().get());
			*log << time() << "No return shell result: " << result << std::endl;
			if (result)
				piconf::error.set(error_t::SYSCALL_ERROR | error_t::PICONFS_ERROR);
			else
				exit(piconf::error.get());
		}
		shell_state |= DONE;
		return false;
	}
	if (shell_state & DONE) {
		return false;
	}
	return true;
}

#endif
