/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _VIO_FD_
#define _VIO_FD_

#include "FD.hpp"

#include <errno.h>
#include <string.h>

VIO::FD::Error::Error(const char* context, const char* message, const void* instance, int code)
: IO::Error(context, message, instance, code) {
}

VIO::FD::FD(int fd)
: IO() {
	if (fd != -1)
		assign(fd);
}

VIO::FD::FD(FILE* stream)
: IO() {
	assign(stream);
}

void VIO::FD::assign(FILE* stream) {
	if (-1 == (this->fd = fileno(stream)))
		throw Error("Cannot map stream pointer to file descriptor", strerror(errno), this, FILENO);
}

void VIO::FD::assign(int fd) {
	this->fd = fd;
	if (!isValid())
		throw Error("Invalid file descriptor assigned", strerror(errno), this, FCNTL);
}

#endif
