/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2013  Rouven Spreckels  <nevuor@piconf.org>                 *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _VIO_FILE_H
#define _VIO_FILE_H

#include "IO.hpp"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define FILE_PATH_DELIMITER "/"

namespace VIO {
	class File;
}

class VIO::File : public IO {
protected:
	typedef char State;
	State state;
	static const State
	OPENED = 0x01;
	std::string file;
public:
	typedef uint8_t error_t;
	static const error_t
	OPEN   = 11,
	STAT   = 12,
	FSTAT  = 13,
	REMOVE = 14,
	MKDIR  = 15;
	class Error : public IO::Error {
	public:
		Error(const char* context, const char* message = 0, const void* instance = 0, int code = 0);
	};
	File(const char* file = 0);
	void assign(const char* file);
	static std::string basename(std::string file);
	static std::string dirname(std::string file);
	static void mkdir(const char* dir);
	void mkdir();
	void open(int flags = O_RDWR, mode_t mode = S_IRUSR | S_IWUSR);
	bool isOpen();
	static bool exists(const char* file);
	bool exists();
	static void remove(const char* file);
	void remove();
	size_t size();
	static size_t size(const char* file);
	void clear();
};

#endif
