/******************************************************************************
 *  This file is part of piconf.                                              *
 *                                                                            *
 *  Copyright (C) 2012, 2013  Rouven Spreckels  <nevuor@piconf.org>           *
 *                                                                            *
 *  piconf is free software: you can redistribute it and/or modify            *
 *  it under the terms of the GNU Affero General Public License version 3 as  *
 *  published by the Free Software Foundation on 19 November 2007.            *
 *                                                                            *
 *  piconf is distributed in the hope that it will be useful,                 *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 *  GNU Affero General Public License for more details.                       *
 *                                                                            *
 *  You should have received a copy of the GNU Affero General Public License  *
 *  along with piconf.  If not, see <http://www.gnu.org/licenses/>.           *
 ******************************************************************************/

#ifndef _VIO_IO_H
#define _VIO_IO_H

#include <set>
#include <vector>
#include <exception>
#include <string>
#include <time.h>
#include <inttypes.h>

namespace VIO {
	class IO;
}

class VIO::IO {
public:
	typedef std::set<IO*> IOS;
protected:
	static void initialize();
	static IOS ios, * iox;
	timespec* time, last;
	const void* that;
	size_t readChunk, writeChunk;
	int fd;
	typedef char Select;
	Select select;
	static const Select
	READABLE = 0x01,
	WRITABLE = 0x02,
	ESPECIAL = 0x04,
	ACTIVITY = 0x08,
	TIMEWISE = 0x10,
	SCHEDULE = 0x20,
	SLEEPING = 0x40;
	class Data {
	protected:
		char* copy;
	public:
		char* data;
		size_t size;
		size_t left;
		Data(const void* data = 0, size_t size = 0, bool copy = false);
		Data(const Data& that);
		Data& operator=(const Data& that);
		void assign(const void* data, size_t size, bool copy);
		~Data();
	} fetched, pushed;
	typedef std::vector<Data> Datas;
	Datas sources, destinations;
	IO(const void* instance = 0);
	virtual void configure();
	virtual void readable();
	virtual void writable();
	virtual void especial();
	virtual void activity();
	virtual void timewise();
	virtual void read();
	virtual void write();
	void terminate(size_t& left);
public:
	typedef uint8_t error_t;
	static const error_t
	GETSOCKOPT =  1,
	SETSOCKOPT =  2,
	READ       =  3,
	WRITE      =  4,
	ESPACIAL   =  5,
	TIMEOUT    =  6,
	SELECT     =  7,
	CLOSE      =  8,
	FORK       =  9,
	BUFFER     = 10,
	TIMER      = 11,
	SIGNAL     = 12;
	static IO* timer;
	class Error : public std::exception {
	public:
		std::string context;
		const void* instance;
		int code;
		Error(const char* context, const char* message = 0, const void* instance = 0, int code = 0);
		virtual const char* what() const throw();
		virtual ~Error() throw();
	};
	class Timeout : public std::exception {
	public:
		int sec, usec;
		Timeout(int sec, int usec);
		virtual const char* what() const throw();
	};
	IO(const IO& io);
	IO& operator=(const IO& io);
	virtual ~IO();
	void owner(const void* that = 0);
	void chunk(size_t readChunk = 0, size_t writeChunk = 0);
	static bool isValid(int fd);
	bool isValid();
	static void mask(IOS* include = 0, IOS* exclude = 0);
	static bool process(int sec = -1, int usec = 0);
	void sleep(bool what);
	void readability(bool what);
	void writability(bool what);
	void fetch(void* data = 0, size_t size = 0, bool string = false);
	void push(const void* data = 0, size_t size = 0, bool copy = false);
	size_t hasToRead(size_t size = 0);
	size_t hasToWrite(size_t size = 0);
	static timespec diff(timespec start, timespec end);
	static char more(timespec more, timespec less);
	void setTimer(IO* timer = 0);
	timespec getTime();
	timespec getLastTime();
	bool isActive(timespec span);
	bool activate(timespec span);
	bool schedule();
	void wait(timespec& time);
	bool hasToWait(timespec time, timespec span);
	void close();
	virtual void clear();
	void swap(IO& io);
};

#endif
